@@ -24,6 +24,9 @@ This module is based on `konto_check <http://kontocheck.sourceforge.net>`_,
 a small library to check German bank accounts. It implements all check
 methods and IBAN generation rules, published by the German Central Bank.
 
+In addition it provides access to the SCL Directory and the possibility
+to verify VAT-IDs by the German Federal Central Tax Office.
+
 Example:
 
 .. sourcecode:: python
@@ -36,18 +39,21 @@ Example:
     bic = kontocheck.get_bic(iban)
 """
 
-import os
+import os, re
+import xmlrpclib
+import sqlite3
 import atexit
 import struct
 from ctypes import c_char_p, c_int, c_void_p, create_string_buffer, \
                     POINTER, byref, CDLL, cdll, util, cast
 c_int_p = POINTER(c_int)
 
-__version__ = '5.5.1'
+__version__ = '5.5.4'
 
 __all__ = ['get_version', 'lut_load', 'lut_is_valid', 'cleanup',
             'check_iban', 'create_iban', 'get_bic', 'get_name',
-            'get_postalcode', 'get_city', 'KontoCheckError']
+            'get_postalcode', 'get_city', 'scl_get_routing',
+            'scl_get_bankname', 'bff_check_vatid', 'KontoCheckError']
 
 
 # Get the path to the default lut file
@@ -67,16 +73,76 @@ libc.free.restype = None
 libc.free.argtypes = (c_void_p,)
 
 
-libname = {
-    'nt': 'kontocheck%s.dll' % _PY_BIT,
-    'os2': 'kontocheck.dylib',
-    }.get(os.name, 'kontocheck.so')
+libname = 'kontocheck' + (os.name == 'nt' and '%s.dll' % _PY_BIT or '.so')
 libpath = os.path.join(_PATH, 'lib', libname)
 if not os.path.isfile(libpath):
     raise RuntimeError('kontocheck library not found')
 
 libkontocheck = CDLL(libpath)
 
+# Connect to SCL database
+sclpath = os.path.join(_PATH, 'data', 'scl.db')
+if not os.path.isfile(sclpath):
+    raise RuntimeError('SCL database not found')
+dbscl = sqlite3.connect(sclpath, check_same_thread=False)
+dbscl.row_factory = sqlite3.Row
+atexit.register(dbscl.close)
+
+def scl_get_routing(bic):
+    """
+    Returns a dictionary with the SEPA routing information for the given
+    BIC from the SCL directory, published by the German Central Bank.
+    """
+    bic = bic.upper()
+    if len(bic) == 8:
+        bic = bic.ljust(11, 'X')
+    cur = dbscl.cursor()
+    cur.execute("SELECT sct, sdd, cor1, b2b FROM scl WHERE bic = ?;", (bic,))
+    result = cur.fetchone()
+    cur.close()
+    if not result:
+        raise ValueError('unknown bic "%s"' % bic)
+    return dict(zip(result.keys(), map(bool, result)))
+
+def scl_get_bankname(bic):
+    """
+    Returns the bank name for the given BIC from the SCL directory,
+    published by the German Central Bank.
+    """
+    bic = bic.upper()
+    if len(bic) == 8:
+        bic = bic.ljust(11, 'X')
+    cur = dbscl.cursor()
+    cur.execute("SELECT name FROM scl WHERE bic = ?;", (bic,))
+    result = cur.fetchone()
+    cur.close()
+    if not result:
+        raise ValueError('unknown bic "%s"' % bic)
+    return result['name']
+
+def bff_check_vatid(local_vatid, remote_vatid, company='', city='',
+                        postalcode='', street='', request_mail=False):
+    """
+    Checks a VAT-ID for validity using the webservice provided by the
+    German Federal Central Tax Office. For further information see
+    https://evatr.bff-online.de/eVatR/xmlrpc/
+    
+    Performs a basic or qualified check depending on the given parameters
+    as described on the following website:
+    https://evatr.bff-online.de/eVatR/xmlrpc/schnittstelle
+    
+    Returns a dictionary with keys as described on the following website:
+    https://evatr.bff-online.de/eVatR/xmlrpc/aufbau
+    
+    The webservice is available daily between 5:00 and 23:00 (CET).
+    """
+    server = xmlrpclib.ServerProxy('https://evatr.bff-online.de/')
+    result = server.evatrRPC(local_vatid, remote_vatid, company, city,
+                        postalcode, street, request_mail and 'ja' or 'nein')
+    result = re.findall('<string>(.*?)</string>', result)
+    result = [v.strip() for v in result]
+    return dict(zip(*[iter(result)] * 2))
+
 
 #~ /* Funktion get_kto_check_version() +§§§1 */
 #~ /* ###########################################################################
@@ -282,7 +348,8 @@ def get_name(bankcode, short_form=False):
 libkontocheck.lut_plz.restype = c_int
 libkontocheck.lut_plz.argtypes = (c_char_p, c_int, c_int_p)
 def get_postalcode(bankcode):
-    """Get the postalcode of a bank by a given bankcode (or German IBAN).
+    """
+    Get the postalcode of a bank by a given bankcode (or German IBAN).
     """
     if bankcode.startswith('DE'):
         bankcode = bankcode[4:12]
